#!/usr/bin/env bash

# ACO SERVER and ACO WEB updater script
#
# parameter:
#           * url where to get the file aco.tar.gz with acoserver.tar.gz and acow-webapp.war inside
#           * md5 code


updater_file="aco.tar.gz"
updater_encrypted_file="aco.tar.gz.gpg"
server_updater_file="acoserver.tar.gz"
web_updater_file="acow-webapp.war"

current_tmp_release_folder=/opt/akkadian/data/tmp
current_app_folder=/home/akkadianuser
current_server_folder=/opt/akkadian/aco-server
current_web_folder=/opt/tomcat/webapps
current_tomcat_base_folder=/opt/tomcat

current_release_zip_name=`echo $1 | sed -r "s/\x1B\[([0-9]{1,3}((;[0-9]{1,3})*)?)?[m|K]//g"`;
jtapi_version=`echo $2`;

# Flags variables
server_flag=0
web_flag=0


isEncrypted=0;
if [[ ( ${current_release_zip_name} == *".gpg" ) ]]
then
    isEncrypted=1;
fi


if [[ -e ${current_tmp_release_folder}/${updater_file} ]]; then
	echo "DEBUG: Deleting ${current_tmp_release_folder}/${updater_file} !" >> /home/akkadianuser/logs/aco_installation.log
	sudo rm -Rf ${current_tmp_release_folder}/${updater_file};
fi

if [[ -e ${current_tmp_release_folder}/${updater_encrypted_file} ]]; then
	echo "DEBUG: Deleting ${current_tmp_release_folder}/${updater_encrypted_file} !" >> /home/akkadianuser/logs/aco_installation.log
	sudo rm -Rf ${current_tmp_release_folder}/${updater_encrypted_file};
fi

echo "File is encrypted: $isEncrypted" > /home/akkadianuser/logs/aco_installation.log

if [[ ( ${current_release_zip_name} == "http"* ) || ( ${current_release_zip_name} == "ftp"* ) ]]
then
    #from CLI
    
    if [[ ( ${isEncrypted} == 1 ) ]]
    then
	    wget $1 -O ${current_tmp_release_folder}/${updater_encrypted_file}
	    if [[ ! -e ${current_tmp_release_folder}/${updater_encrypted_file} ]]; then
		echo "ERROR: Couldn't download updater gpg file!" > /home/akkadianuser/logs/aco_installation.log
		echo "ERROR: Couldn't download updater file!"; exit;
	    fi
    else 
	    wget $1 -O ${current_tmp_release_folder}/${updater_file}
	    if [[ ! -e ${current_tmp_release_folder}/${updater_file} ]]; then
		echo "ERROR: Couldn't download updater file!" > /home/akkadianuser/logs/aco_installation.log
		echo "ERROR: Couldn't download updater file!"; exit;
	    fi
    fi
else
    #from portal    
    if [[ ( ${isEncrypted} == 1 ) ]]
    then
	    cp -Rf ${current_release_zip_name} ${current_tmp_release_folder}/${updater_encrypted_file}
	    if [[ ! -e ${current_tmp_release_folder}/${updater_encrypted_file} ]]; then
		echo "ERROR: Couldn't copy encrypted media file!" > /home/akkadianuser/logs/aco_installation.log
		echo "ERROR: Couldn't copy encrypted media file!"; exit;
	    fi

    else
	    cp -Rf ${current_release_zip_name} ${current_tmp_release_folder}/${updater_file}
	    if [[ ! -e ${current_tmp_release_folder}/${updater_file} ]]; then
		echo "ERROR: Couldn't copy media file!" > /home/akkadianuser/logs/aco_installation.log
		echo "ERROR: Couldn't copy media file!"; exit;
	    fi
    fi
fi




if [[ ( ${isEncrypted} == 1 ) ]]
then
	gpg --batch --passphrase-file=/home/akkadianuser/scripts/security/installationkey --output ${current_tmp_release_folder}/${updater_file} --decrypt ${current_tmp_release_folder}/${updater_encrypted_file} 2>&1 >> /home/akkadianuser/logs/decryption.log
	if [[ ! -e ${current_tmp_release_folder}/${updater_file} ]]; then
		echo "ERROR: Decryption failed for file: ${current_tmp_release_folder}/${updater_file}   ${current_tmp_release_folder}/${updater_encrypted_file}" >> /home/akkadianuser/logs/aco_installation.log
		
		echo "ERROR: Decryption failed for file!"; exit;
	fi
fi



# check=$(awk -F ': ' '{print $2}' <<< $(md5sum -c <<< "$2 ${current_tmp_release_folder}/${updater_file}"))
# if [ "$check" != "OK" ]; then
#    echo "ERROR: no properly formatted MD5 checksum lines found"; exit;
# fi
echo "Starting the installation of Akkadian CO" > /home/akkadianuser/logs/aco_installation.log
date >> /home/akkadianuser/logs/aco_installation.log

tar -xvzf ${current_tmp_release_folder}/${updater_file} -C ${current_tmp_release_folder}
if [[ ! -e ${current_tmp_release_folder}/${server_updater_file} ]]; then
    if [[ ! -e ${current_tmp_release_folder}/${web_updater_file} ]]; then
        echo "ERROR: The downloaded files is wrong. Please try again!" >> /home/akkadianuser/logs/aco_installation.log
        exit;
    else
        echo "WARNING: The downloaded file don't contains the server updater. Only the ACO WEB will be update!" >> /home/akkadianuser/logs/aco_installation.log
        web_flag=1
    fi
else
    if [[ ! -e ${current_tmp_release_folder}/${web_updater_file} ]]; then
        echo "WARNING: The downloaded file don't contains the web updater. Only the ACO SERVER will be update!" >> /home/akkadianuser/logs/aco_installation.log
        server_flag=1
    else
        server_flag=1
        web_flag=1
    fi
fi

echo "server flag=${server_flag} web flag=${web_flag}" >> /home/akkadianuser/logs/aco_installation.log


if [[ "${server_flag}" -eq 1 ]]; then
    echo $'\n' "Updating ACO SERVER..." >> /home/akkadianuser/logs/aco_installation.log

    sudo yes | cp -Rf /opt/akkadian/aco-server/certificates/aco-server.crt /opt/akkadian/data/tmp/aco-server.crt
    sudo yes | cp -Rf /opt/akkadian/aco-server/certificates/aco-server.key /opt/akkadian/data/tmp/aco-server.key
    if [[ -f /opt/akkadian/aco-server/certificates/aco-server.csr ]];
    then
    sudo yes | cp -Rf /opt/akkadian/aco-server/certificates/aco-server.csr /opt/akkadian/data/tmp/aco-server.csr
    fi
    

    # in case mongo is set with replica
    sudo yes | cp -Rf /opt/akkadian/aco-server/scripts/start-mongodb.sh /opt/akkadian/data/tmp/start-mongodb.sh

    # sudo sh /home/akkadianuser/scripts/stop_kestrel.sh
    echo "Extracting  ${current_tmp_release_folder}/${server_updater_file}  to ${current_server_folder} " 
    sudo tar -xvzf ${current_tmp_release_folder}/${server_updater_file} -C ${current_server_folder}/ --strip-components 1
    dos2unix ${current_server_folder}/scripts/*
    dos2unix ${current_server_folder}/configs/*
    sudo chmod -R 765 /opt/akkadian/

    #in case we have replica
    sudo chmod 600 /opt/akkadian/data/mongokey

    echo $'\n' "Copying config files..." >> /home/akkadianuser/logs/aco_installation.log

    sudo yes | cp -Rf /opt/akkadian/aco-server/configs/sysctl.conf /etc/sysctl.conf
    sudo yes | cp -Rf /opt/akkadian/aco-server/configs/limits.conf /etc/security/limits.conf
    sudo yes | cp -Rf /opt/akkadian/aco-server/configs/supervisord.conf /etc/supervisord.conf
    sudo yes | cp -Rf /opt/akkadian/aco-server/configs/aco-server.service /etc/systemd/system/

    sudo yes | cp -Rf /opt/akkadian/aco-server/configs/nginx.conf /etc/nginx/
    sudo yes | cp -Rf /opt/akkadian/aco-server/configs/40x.html   /usr/share/nginx/html/
    sudo yes | cp -Rf /opt/akkadian/aco-server/configs/50x.html   /usr/share/nginx/html/
    sudo yes | cp -Rf /opt/akkadian/aco-server/configs/404.html   /usr/share/nginx/html//
    sudo yes | cp -Rf /opt/akkadian/aco-server/configs/aCO-icon.png   /usr/share/nginx/html/
    sudo yes | cp -Rf /opt/akkadian/aco-server/configs/aCO_Logo_big.svg  /usr/share/nginx/html/


    #####  getting environment type, it can be empty if it is an old file
    echo "getting environment type, it can be empty if it is an old file" >> /home/akkadianuser/logs/aco_installation.log
    env=$(sudo cat /opt/tomcat/conf/fp-akkadian-platform.properties | grep aco.web.environment.size | awk -F "=" '{print $2}')
    sudo yes | cp -Rf ${current_server_folder}/acow-installer/fp-akkadian-platform.properties ${current_tomcat_base_folder}/conf/
    ####  setting environment type
    if [[ (-z "$env") || ("$env" == " ") || ("$env" == "\n") ]]
    then
        echo "$env is empty" >> /home/akkadianuser/logs/aco_installation.log
    else
        sudo sed -i "s/aco.web.environment.size.*/aco.web.environment.size=$env/g" /opt/tomcat/conf/fp-akkadian-platform.properties
    fi

    sudo yes | cp -Rf ${current_server_folder}/acow-installer/server.xml ${current_tomcat_base_folder}/conf/

    echo "getting jtapi from: $2" >> /home/akkadianuser/logs/aco_installation.log
    sudo /usr/bin/wget $2 -O /home/akkadianuser/jtapi.jar --no-check-certificate
    sudo yes | cp -Rf /home/akkadianuser/jtapi.jar ${current_tomcat_base_folder}/lib/jtapi-9.1.jar
    sudo rm -Rf /home/akkadianuser/jtapi.jar
    sleep 5
    sudo yes | cp -Rf ${current_server_folder}/acow-installer/jtapi.ini ${current_tomcat_base_folder}/lib/

    echo "getting /etc/systemd/system/tomcat.service memory and save it" >> /home/akkadianuser/logs/aco_installation.log
    memory=$(sudo cat /etc/systemd/system/tomcat.service | grep "Environment='CATALINA_OPTS")
    sudo yes | cp -Rf ${current_server_folder}/configs/tomcat.service /etc/systemd/system/
    #setting memory into new file
    sudo sed -i "s/Environment="\'"CATALINA_OPTS.*/$memory/g" /etc/systemd/system/tomcat.service

    sudo yes | cp -Rf ${current_server_folder}/configs/my.cnf /etc/
    sudo yes | cp -Rf ${current_server_folder}/configs/mysqld.service /usr/lib/systemd/system/

    echo $'\n' "reloading daemon..." >> /home/akkadianuser/logs/aco_installation.log
    sudo systemctl daemon-reload

    echo "changing permissions over the replica file" >> /home/akkadianuser/logs/aco_installation.log
    if [[ ! -f /opt/akkadian/data/replica ]];
    then
        sudo echo " " > /opt/akkadian/data/replica
        sudo chmod 666 /opt/akkadian/data/replica
    else
        sudo chmod 600 /opt/akkadian/data/mongokey
        sudo chmod 666 /opt/akkadian/data/replica
    fi

    echo "Restoring certificates" >> /home/akkadianuser/logs/aco_installation.log

    sudo yes | cp -Rf /opt/akkadian/data/tmp/aco-server.crt /opt/akkadian/aco-server/certificates/aco-server.crt
    sudo yes | cp -Rf /opt/akkadian/data/tmp/aco-server.key /opt/akkadian/aco-server/certificates/aco-server.key
    if [[ -f /opt/akkadian/data/tmp/aco-server.csr ]];
    then
        sudo yes | cp -Rf /opt/akkadian/data/tmp/aco-server.csr /opt/akkadian/aco-server/certificates/aco-server.csr
    fi

    # in case mongo is set with replica
    sudo yes | cp -Rf /opt/akkadian/data/tmp/start-mongodb.sh /opt/akkadian/aco-server/scripts/start-mongodb.sh

fi


if [[ "${web_flag}" -eq 1 ]]; then

    echo $'\n' "Updating ACO WEB..." >> /home/akkadianuser/logs/aco_installation.log

    sudo yes | cp -Rf /opt/tomcat/webapps/acow-webapp/WEB-INF/applicationContext-mongo.xml /opt/akkadian/data/tmp/applicationContext-mongo.xml
    sudo sh /home/akkadianuser/scripts/stop_tomcat.sh

    sudo rm ${current_web_folder}/${web_updater_file}
    sudo rm -Rf ${current_web_folder}/acow-webapp

    sudo cp ${current_tmp_release_folder}/${web_updater_file} ${current_web_folder}/

    sudo mysql --defaults-extra-file=/home/akkadianuser/.my_aco.cnf -e "DROP DATABASE fp;"
    sudo mysql --defaults-extra-file=/home/akkadianuser/.my_aco.cnf -e "CREATE DATABASE fp;"

    # CW tunning up
    # sudo cp -Rf /etc/systemd/system/multi-user.target.wants/mysqld.service
    # sudo cp -Rf /home/akkadianuser/upgrades/aco_system_mysql_cnf/my.cnf /etc/my.cnf
    # sudo systemctl restart mysqld

    sudo chmod -R 0666 /opt/tomcat/logs
    sudo rm -rf /opt/tomcat/logs/*

    #sudo sh /home/akkadianuser/scripts/start_tomcat.sh
    # In case mongo is part of a replica
    sudo yes | cp -Rf /opt/akkadian/data/tmp/applicationContext-mongo.xml  /opt/tomcat/webapps/acow-webapp/WEB-INF/applicationContext-mongo.xml

fi

if [[ ! -d /opt/akkadian/aco-server/external_certificates ]]; then
    sudo mkdir /opt/akkadian/aco-server/external_certificates
fi

echo $'\n' "Cleaning..." >> /home/akkadianuser/logs/aco_installation.log
sudo rm -Rf ${current_tmp_release_folder}/*
echo $'\n' "DONE!" >> /home/akkadianuser/logs/aco_installation.log
sleep 3

echo  "Restarting aco server" >> /home/akkadianuser/logs/aco_installation.log
sudo systemctl stop aco-server
sleep 1

#install net 5 if needed
current_version=$(dotnet --info | grep Version: | head -n 1 | awk '{print $2}')
line=$(cat /opt/akkadian/aco-server/wwwroot/version.js)
aco_version=$(printf $line)
aco_version_needs_update=$(python /home/akkadianuser/version_comparison.py gt $aco_version "4.9.1")

if [[ "$current_version" == "2.1.1" && "$aco_version_needs_update" == "True" ]]; then
    sudo mkdir -p /home/akkadianuser/dotnet
    sudo tar zxf /home/akkadianuser/packages/dotnet-sdk-5.0.400-linux-x64.tar.gz -C /home/akkadianuser/dotnet
    export DOTNET_ROOT=/home/akkadianuser/dotnet
    export PATH=$PATH:/home/akkadianuser/dotnet
    printf DOTNET_ROOT=/home/akkadianuser/dotnet >> /home/akkadianuser/.bashrc
    printf PATH=$PATH:/home/akkadianuser/dotnet >> /home/akkadianuser/.bashrc
    sudo sed -i "s,ExecStart=/usr/bin/dotnet,ExecStart=/home/akkadianuser/dotnet/dotnet,g" /etc/systemd/system/aco-server.service
    echo ".net 5 installation done" >> /home/akkadianuser/logs/aco_installation.log
    sudo systemctl daemon-reload
fi

akkadian_cron_content=$(sudo crontab -u akkadianuser -l)
if [[ "$akkadian_cron_content" != *"remove_sch_backups.sh"* ]]; then
    akkadian="0 1 * * * sh /home/akkadianuser/scripts/aco_utils/remove_sch_backups.sh > /home/akkadianuser/logs/aco_installation.log 2>&1"
    (sudo crontab -u akkadianuser -l; echo "$akkadian" ) | sudo crontab -u akkadianuser -
fi

if [[ "$akkadian_cron_content" != *"remove_logs.sh"* ]]; then
    akkadian="0 0 * * * sh /home/akkadianuser/scripts/aco_utils/remove_logs.sh > /home/akkadianuser/logs/aco_installation.log 2>&1"
    (sudo crontab -u akkadianuser -l; echo "$akkadian" ) | sudo crontab -u akkadianuser -
fi


# upgrade nginx if needed
sudo sh /home/akkadianuser/scripts/upgrade_nginx/upgradeNginx.sh


#remove net 2
#update aco-server service

sudo systemctl start aco-server
sleep 2

echo "Restarting tomcat" >> /home/akkadianuser/logs/aco_installation.log
sudo systemctl restart tomcat
sleep 1

echo "Restarting nginx" >> /home/akkadianuser/logs/aco_installation.log
sudo systemctl restart nginx
