
from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer

from cdr_db_handler import CDRDBHandler
from cdr_file_handler import CDRFileHandler


def init():
    """Loads up the config and returns the values. Also checks for and creates tracking table if it doesnt exist.

    Raises:
        e: General exception raised when error occurs

    Returns:
        config (dictionary): The config paramters in a dictionary 
    """
    try: 
        print('Starting Akkadian CDR Data Collection Service',flush=True)
        config = CDRDBHandler.loadConfig() # test config loading.
        tracking_exists = CDRDBHandler.check_table_exists('akk_cdr_tracking')
        if tracking_exists == False:
            CDRDBHandler.create_tracking_table()
        return config
    except Exception as e:
        print('Init failed.',flush=True)
        raise e
    

if __name__ == "__main__":
    try:
        config = init()
        path = config['cdr_location']
        event_handler = CDRFileHandler()
        observer = Observer()
        observer.schedule(event_handler, path, recursive=False) # watch folder for file creations
        observer.start()

        while observer.isAlive():
            observer.join(1)
    except KeyboardInterrupt:
        print('Data Collector stopped. Exiting',flush=True)
        observer.stop()
    except Exception as e:
        print(f"{type(e).__name__} at line {e.__traceback__.tb_lineno} of {__file__}: {e}",flush=True)
        observer.stop()
        exit(1)
    observer.join()
