#! /bin/bash
# Installs the Akkadian Service Monitoring Tool

ASM_INSTALL_DIRECTORY="/usr/local/sbin/asm"

# Create web directory
VERSION=$(jq ".version" < ./config.json | tr -d '"')
FOLDER_NUMBER="v${VERSION%${VERSION#?}}" 
ASM_WEB_DIRECTORY="/var/www/html/asm/${FOLDER_NUMBER}/" 

echo "Installing Akkadian Service Monitoring Tool ${FOLDER_NUMBER}"


# create folders if they dont exist
if [[ ! -d "${ASM_WEB_DIRECTORY}" ]]; then
    mkdir -p "${ASM_WEB_DIRECTORY}"
elif [[ ! -d "${ASM_INSTALL_DIRECTORY}" ]]; then
    echo "Creating ASM installation directory"
    mkdir -p "${ASM_INSTALL_DIRECTORY}"
fi 

# copy files to install directory
\cp ./akkadian_sm.sh "${ASM_INSTALL_DIRECTORY}"/akkadian_sm.sh
\cp ./default_services "${ASM_INSTALL_DIRECTORY}"/default_services
\cp ./asm_services_location "${ASM_INSTALL_DIRECTORY}"/asm_services_location
\cp ./config.json "${ASM_INSTALL_DIRECTORY}"/config.json
\cp ./services_file_template.json "${ASM_INSTALL_DIRECTORY}"/services_file_template.json


# copy web files 
\cp ./web_files/.htaccess "${ASM_WEB_DIRECTORY}/.htaccess"


# Set file permissions
chmod 0770 "${ASM_INSTALL_DIRECTORY}"/* 
chown root:root "${ASM_INSTALL_DIRECTORY}"/akkadian_sm.sh
chown root:root "${ASM_INSTALL_DIRECTORY}"/default_services

# copy service files to systemd
echo "Copying service files"
cp ./akkadian_sm.service /etc/systemd/system/akkadian_sm.service
cp ./akkadian_sm.timer /etc/systemd/system/akkadian_sm.timer

# start service
echo "Starting service"
systemctl start akkadian_sm

# enable on boot
echo "Enabling service"
systemctl enable akkadian_sm

# reload systemd config
systemctl daemon-reload
echo "Reload Complete"